% stst = fctn_bool_single(ppar,step,ttab,sbeg,sstp,spar)
%
% function to simulate stressosome dynamics.
%
% input:
%   ppar: phosphorylation parameter
%     kphr = 1; % phosphorylation propensity for R
%     kdpr = .4;%vdpr(i0);%1; % dephosphorylation propensity for R
%     kphs = .2; % phosphorylation propensity for S
%     kdps = 1;%vdps(i0); % dephosphorylation propensity for S
%   step: simulation steps
%   sbeg: vector with times where stress begins
%   sstp: vector with times where stress stops
%   spar: vector with values of the parameter that changes with stress
%   ttab: table to determine parameters of protein cooperativity with
%   respect to phosphorylation
%
% output:
%   stst: matrix with step number of columns (simulation steps) and 60 rows
%   (number of proteins). At each time step a protein is either
%   phosphorylated or not (1,0).
%
% procedure:
%   The stressosome is initiated with a random distribution of
%   phosphorylations yielding a phosphorylation of half the proteins of
%   R/S. At each time step a random list is generated that determines the
%   update succession for the proteins. For each protein the
%   phosphorylation of the neighbours are defined based on the four
%   distinct neighbourhoods. For each neighbourhood, and for each
%   phosphorylation distribution there exist a value in a table that
%   determines how the likelyhood of a phosphorylation is changed by
%   protein cooperativity. Thus, testing different protein cooperativity
%   schemas is done by changing the values of those tables.
%   Dephosphorylation occurs with a defined probability.
%
%
% date: 11/05/09
% modification date: 11/05/09
%
% author: Ulf Liebal
% contact: ulfliebal@googlemail.com
% 
% please cite the original.


function stst = fctn_bool_single(ppar,step,ttab,sbeg,sstp,spar)

%% number of positions, list length
posi = 1:60;

[t1a,t2a,t3a,t4a] = ttab();

% coordinates of t neighbourhoods
ct1a = [5,10,13,14,15,16,17,18,21,32,29,40,43,44,45,46,47,48,51,56];

ct2a1 = [1,4,6,9,11,12,23,24,25,26,27,30];
ct2a2 = sort(61-ct2a1);
ct2a = [ct2a1,ct2a2];

ct3a = [7,8,20,22,39,41,53,54];

ct4a = [2,3,19,28,33,42,58,59];

% connectivity table, first entry shows unique neighbour, i.e. either R in
% RSS or S in SRR
cts1 = [10,4,1;5,11,20;14,6,22;13,7,23;16,7,24;15,8,25;18,8,26;17,9,27;32,12,22;21,31,41]; 
ctn1 = 61-cts1;
cta1 = [cts1;[ctn1([11-(1:10)],:)]]; % t1a connections, dimer partners are first entry

cts2 = [5,2,6;5,9,3;13,12,1;18,19,4;10,30,12;21,6,11;14,24,34;15,23,35;16,26,36;17,25,37;18,28,38;29,11,31];
ctn2 = 61-cts2;
cta2 = [cts2;[ctn2([13-(1:12)],:)]]; % t2a connections, dimer partners are last entry

cts3 = [2,14,15;3,16,17;19,10,29;33,13,21];
ctn3 = 61-cts3;
cta3 = [cts3;[ctn3([5-(1:4)],:)]]; % t3a connections, dimer partners are first entry

cts4 = [7,1,3;8,2,4;20,9,28;39,27,19];
ctn4 = 61-cts4;
cta4 = [cts4;[ctn4([5-(1:4)],:)]]; % t4a connections, dimer partners are first entry

ctab(ct1a,:) = cta1;
ctab(ct2a,:) = cta2;
ctab(ct3a,:) = cta3;
ctab(ct4a,:) = cta4;
%cent = [ct1a,ct2a,ct3a,ct4a]';

% defining positions of RsbS
sprs = [5,10,13,14,15,16,17,18,21,32]; % RsbS protein south
sprn = sort(61-sprs);
rsbs = [sprs,sprn];

rsbr = setdiff([1:60],rsbs);

% sidx = 1:20;
% ridx = 21:60;
% r1dx = 21:44;
% r2dx = 45:52;
% r3dx = 53:60;

clear stst
% random initialization of stst
stst(:,1) = floor(2*rand(60,1));
%stst = zeros(60,1);

for i1 = 1:step
    kphr = ppar(1); % phosphorylation propensity for R
    kdpr = ppar(2);%vdpr(i0);%1; % dephosphorylation propensity for R
    kphs = ppar(3); % phosphorylation propensity for S
    kdps = ppar(4);%vdps(i0); % dephosphorylation propensity for S

    if exist('spar','var')
        for i3 = 1:length(sbeg)
            if i1 >= sbeg(i3) && i1 < sstp(i3)
                kphr = spar;
            end
        end
    end
	%t3a(4,3)
	astr = stst(:,end);
	% neighbourhood state
	nsta = astr(ctab);
	
	pnet(:,1) = nsta(:,1); % first neighbour, either single R or S
	pnet(:,2) = sum(nsta(:,[2:3]),2); % phosphorylation of two neighbours either 2R or 2S
	pnet(:,3) = pnet(:,1)+pnet(:,2);

	% first we make phosphorylations
	% we need to find the non-phosphorylated positions
	sphi = rsbs(logical(stst(rsbs,i1)))'; % S phosphorylation index
	snph = setdiff(rsbs,sphi)'; % these are positions of S not phosphorylated
	rphi = rsbr(logical(astr(rsbr)))'; % R phosphorylation index
	rnph = setdiff(rsbr,rphi)'; % these are positions of R not phosphorylated
	
	rnp2 = intersect(rnph,ct2a)'; % non-phosphorylated r with neighb. S-R-R
	rnp3 = intersect(rnph,ct3a)'; % non-phosphorylated r with R-S-S
	rnp4 = intersect(rnph,ct4a)'; % non-phosphorylated r with R-R-R
	
	for i2 = 1:size(snph,1) % determining future of non-phosph RsbS proteins
		tinta = find(pnet(snph(i2),1)==t1a(:,1));
		tintb = find(pnet(snph(i2),2)==t1a(:,2));
		tint = intersect(tinta,tintb);
		phor = floor(t1a(tint,3).*kphs+rand); % vector showing new phosphorylation events
        stst(snph(i2),i1+1) = phor;
	end
	
	for i2 = 1:size(rnp2,1) % determining future of non-phosph RsbR proteins, of neigbourhood ct2a
		tinta = find(pnet(rnp2(i2),1)==t2a(:,1));
		tintb = find(pnet(rnp2(i2),2)==t2a(:,2));
		tint = intersect(tinta,tintb);
		phor = floor(t2a(tint,3).*kphr+rand); % vector showing new phosphorylation events
        stst(rnp2(i2),i1+1) = phor;
    end
    
	for i2 = 1:size(rnp3,1) % determining future of non-phosph RsbR proteins, of neigbourhood ct2a
		tinta = find(pnet(rnp3(i2),1)==t3a(:,1));
		tintb = find(pnet(rnp3(i2),2)==t3a(:,2));
		tint = intersect(tinta,tintb);
		phor = floor(t3a(tint,3).*kphr+rand); % vector showing new phosphorylation events
        stst(rnp3(i2),i1+1) = phor;
    end
  
	for i2 = 1:size(rnp4,1) % determining future of non-phosph RsbR proteins, of neigbourhood ct2a
		tint = find(pnet(rnp4(i2),3)==t4a(:,1));
		phor = floor(t4a(tint,2).*kphr+rand); % vector showing new phosphorylation events
        stst(rnp4(i2),i1+1) = phor;
    end
	
	% now we turn to the already phosphorylated sites and determine whether
	% they get dephosphorylated in the next step. This is done by a random
	% uniform distribution
	dphr = floor((1-kdpr)+rand(length(rphi),1)); % here we have to calculate with 1-kdph because, well, there is a good reason, write me for the complete answer. In short, zeros represent dephosphorylation.
	dphs = floor((1-kdps)+rand(length(sphi),1));
	stst(rphi,i1+1) = dphr;
	stst(sphi,i1+1) = dphs;
     
end

