%% This function simulates the activation of the stressosome based on 
%  hypothesis derived from a paper model. In the paper model is was
%  observed that dimers of RsbS and RsbR form triangles that can collapse.
%  A collapse results in the inhibition of neighbouring triangles as each
%  dimer is associated with two triangles. Once it decides for one triangle
%  to collapse the other triangle of which it is part cannot collapse any more. 
%
% colm = trianglestressosome_1107(ppar,step,strin)
% ppar: parameter vector, order rpho,rdph,coop
% step: number of simulation steps
% strin:stress inpunt, new values for stress parameter, first scalar
%       represents the step at which stress starts second parameter is a
%       new parameter.

function colm = trianglestressosome_1107(ppar,step,strin)%,ttab,sbeg,sstp,spar)
tria = 1:20;
% triangles
tnei = [2,5,6;1,3,7;2,4,8;3,5,9;1,4,10;1,11,15;2,11,12;3,12,13;4,13,14;5,14,15;...
         6,7,16;7,8,17;8,9,18;9,10,19;6,10,20;11,17,20;12,16,18;13,17,19;14,18,20;15,16,19];
% neigbhouring triangles for each triangle in order of their position

if ~exist('strin','var')
    strin(1) = step;
    strin(2) = ppar(3);
end    
% parameters
rpho = ppar(1); % exterial controlled phosphorylation/collapse probability
rdph = ppar(2); % probability that a dephosphorylation/relaxation occurs

stps = step;
tsta = zeros(20,stps+1);
neip = reshape(tsta(tnei,1),20,3);
% state of neighbouring triangles
for i1 = 1:stps
    if i1 < strin(1)
        coop = ppar(3);
        qest = (1-ppar(1))*coop; % qest is a probability that increases the phosphorylation by the remainder of rpho. (1-rpho)/3; % activation by collapse by inhibited triangles, if all three neighbouring triangles are inhibited then the probability of collapse is 1.
    else
        coop = strin(2);
        qest = (1-ppar(1))*coop; % qest is a probability that increases the phosphorylation by the remainder of rpho. (1-rpho)/3; % activation by collapse by inhibited triangles, if all three neighbouring triangles are inhibited then the probability of collapse is 1.        
    end
    ulst = randperm(20)';
    
  % three possible states exist for any triangle:
  % (i)   0: dimers of the triangle are not phosphorylated
  % (ii)  1: collapsed triangle, dimers are phosphorylated
  % (iii) 2: the triangle is partly collapsed, i.e. a dimer of the triangle
  %          bended to another triangle. This triangle cannot collapse any
  %          more but it may (hypothesis) stimulate collapse of
  %          neighbouring triangles.
  for i2 = 1:length(ulst)
      if tsta(ulst(i2),i1) == 0
          % does a phosphorylation occur?:
          inhn = find(neip(ulst(i2),:)==2);
          phos = rpho+length(inhn)*qest>rand;
          if phos==1
              % if a reaction occurs the triangle collapses and the
              % neighbouring triangles are inhibited to collapse.         
              tsta(ulst(i2),i1)=phos;
              % collapse of the current triangle
              tsta(tnei(ulst(i2),:),i1)=2;
              % inhibiting neighbouring triangles
              neip = reshape(tsta(tnei,i1),20,3);
          end
      elseif tsta(ulst(1),i1) == 1
          % does a dephosphorylation occur?
          deph = rdph>rand; % deph = 1 corresponds to a dephosphorylation event
          % if a dephosphorylation occurs then the neighbouring triangles
          % may have the possibility to collapse but only if the neighbours
          % of the neighbours are not already collapsed.
          if deph == 1
              tsta(ulst(i2),i1) = 0;
              % relaxation of the current triangle
              neip = reshape(tsta(tnei,i1),20,3);
          end
      end
  end
  % we have to check whether all neighbours of collapsed triangles are
  % inhibited (set to 2).
  [colp,b]=find(tsta(:,i1)==1); % positions that are collapsed 
  for i2 = 1:length(colp)
      tsta(tnei(colp(i2),:),i1)=2;
  end
  % we have to check whether inhibited triangles are still inhibited or
  % whether dephosphorylation reactions freed a triangle
  [inhp,b]=find(tsta(:,i1)==2); % positions that should have collapsed neighbours
  % if none of the neighbours of inhibited positions (2) is collapsed (1)
  % then the position is set to relaxed (1).
  for i2 = 1:length(inhp)
      if isempty(find(tsta(tnei(inhp(i2),:),i1)==1))
          tsta(inhp(i2),i1)=0;
      end
  end
  tsta(:,i1+1)=tsta(:,i1);
end
tsta(:,end)=[];
cols = find(tsta==1);
colm = zeros(size(tsta));
colm(cols)=1;